// Convodution back projection method by Tsukasa Nakano, GSJ
// Required files are sinogram data, sino.tmp.

/*----------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <time.h>
#include "cbp.h"
//#include <malloc.h>
 
/*----------------------------------------------------------------------*/
#define INTEL

/*----------------------------------------------------------------------*/

// main data for read transmitted images (data[y][x])
//short	data[MAXPIXL][MAXPIXL];
double	*prj_temp;

// output file name
char	fn[15];

// region of calculation in raw data(transmitted images)
short	x1, x2, z0, z1;


// common parameter
int		NST;
double	r0;

// total absorption
double	TAM, TASD;


/*----------------------------------------------------------------------*/

static void Error(msg)
char        *msg;
	{
		fputs(msg,stderr);
		fputc('\n',stderr);
		exit(1);
	}

/*----------------------------------------------------------------------*/

#ifndef CLOCKS_PER_SEC
#define CLOCKS_PER_SEC	1000000
extern long clock();
#endif

#define CLOCK()		((double)clock()/(double)CLOCKS_PER_SEC)
#define DESC_LEN	256

int main(argc,argv)
int		argc;
char	**argv;
{
	short		N;				// reconstructed image full size (N*N)
	double		N12;			// N12 = (N-1)/2
	short		ln;				// layer
	short		M;				// number of projection

	double		Clock;			// timer setting
	Float		**P, **f;			// full size of reconstructed image

	int			vv, hh;			// 
	double		delta, theta0;
	double		*rec_temp;
	int			i_tmp;			// temporary parameter for integer
	FILE		*fp;
	char		lne[20], *ss;
	long i,j;

	if (argc>3)
		Error(" usage : ct_cbp (theta0)\n");

//open input files
	if((fp = fopen("sino.tmp","r")) == NULL){ 
		printf("can not open sino.tmp for input\n");
		return(1);
	}
	lne[0]='\0'; ss=fgets(lne, 20, fp); N=atoi(lne);
	lne[0]='\0'; ss=fgets(lne, 20, fp); M=atoi(lne);
	lne[0]='\0'; ss=fgets(lne, 20, fp); ln=atoi(lne);
	lne[0]='\0'; ss=fgets(lne, 20, fp); r0=atof(lne);
	lne[0]='\0'; ss=fgets(lne, 20, fp); TAM=atof(lne);
	lne[0]='\0'; ss=fgets(lne, 20, fp); TASD=atof(lne);
	fclose(fp);

	sprintf(fn,"s%04d.sin", ln);

	printf("calclation layer\t: %d\n", ln);
	printf("sinogram file name\t: %s\n", fn);
	printf("Width of image\t\t: %d\n", N);
	printf("Number of projection\t: %d\n", M);
	printf("Rotation center\t\t: %f\n", (float)r0);

// Initialize
//	Clock=CLOCK();
	if ((P=InitCBP(N,M))==NULL) Error("memory allocation error.");
//	fprintf(stderr,"SetUpCBP\t\t%lf / sec\n",CLOCK()-Clock);

//open input files
	if((fp = fopen(fn,"rb")) == NULL){
		printf("can not open %s for input\n", fn);
		return(-1);
	}
	prj_temp = (double *)malloc(N*M*sizeof(double));
	if ((i = fread(prj_temp , sizeof(double), N*M, fp))!=N*M){
		printf("error in reading sinogram (%d)\n", (int)i);
		return(-2);
	}
	fclose(fp);
	for (j=0; j<M; j++){
		for (i=0; i<N; i++){
//			if(abs(*(prj_temp+N*j+i))<100){
				P[j][i]=*(prj_temp+N*j+i);
//			}
		}
	}

//
	r0=-1.0*r0;
	delta = 1.0; //normilized data

	N12		= (double)(N-1) / 2.0;
	if (argc==2){
		theta0 = M_PI/180.0*atof(argv[1]);
	} else{
		theta0 = 0.0;
	}

// Convolution ̏v
	Clock=CLOCK();
	f=CBP(delta,r0,theta0);

	rec_temp= (double *)malloc(N*N*sizeof(double));

	for (vv=0; vv<N; vv++){
		for (hh=0; hh<N; hh++){
			*(rec_temp+N*vv+hh) = f[vv][hh];
		}
	}
	fprintf(stderr,"reconstruction\t\t: %lf / sec\n",CLOCK()-Clock);

// output reconstructed data
	sprintf(fn,"r%04d.rec", ln);
	if((fp = fopen(fn,"wb")) == NULL){ 
		printf("can not open %s for output\n", fn);
		return(-3);
	}

	hh=0;
	for (vv=0;vv<N;vv++){
		for (hh=0;hh<N;hh++){
			if((i_tmp = fwrite((rec_temp+N*vv+hh) , sizeof(double), 1, fp))!=1){
				printf("error writing rec data (%d)", i_tmp);
				return (-4);
			}
		}
	}

	fclose(fp);

// output the detail of calculation

	if((fp = fopen("rec.tmp","w")) == NULL){ 
		printf("can not open rec.tmp for output\n");
		return(-4);
	}
	fprintf(fp, "%d\n", ln);
	fprintf(fp, "%d\n", N);
	fprintf(fp, "%d\n", M);
	fprintf(fp, "%d\n", N);
	fprintf(fp, "%lf\n", r0);
//	fprintf(fp, "%d\n", ifilter);
	fprintf(fp, "%d\n", 1); // HAN tB^[cheslar^CvɌŒB
	fprintf(fp, "%lf\n", TAM); // Mean of Total Absorption from sinogram
	fprintf(fp, "%lf\n", TASD); // SD of Total Absorption from sinogram
	fclose(fp);

// free memory
	TermCBP();
	
// remove sinogram file

	sprintf(fn,"s%04d.sin", ln);
	if(remove( fn ) == -1 ){
      printf( "%s could not be deleted.",fn );
	}

	return (0);
}
